import django
from django.db.models import Manager


def QuerySetManager(qs):
    class QuerySetManager(Manager):
        qs_class = qs

        def get_queryset(self):
            return self.qs_class(self.model, using=self._db)

        def get_restore_or_create(self, *args, **kwargs):
            return self.get_queryset().get_restore_or_create(*args, **kwargs)

        def restore(self, *args, **kwargs):
            return self.get_queryset().restore(*args, **kwargs)
    return QuerySetManager()


def MultiPassThroughManager(*classes):
    name = "".join([cls.__name__ for cls in classes])
    if django.VERSION < (1, 7, 0):
        from model_utils.managers import PassThroughManager
        result_class = PassThroughManager.for_queryset_class(type(name, classes, {}))
        result = result_class()

    else:
        result_class = type(name, classes, {})
        result = result_class.as_manager()

    globals()[name] = result_class

    return result



import copy
from functools import partial

import django
from django.db.models.deletion import Collector
from django.db.models.query import QuerySet

from django.db.models.query_utils import Q
from django.db.models.sql.where import WhereNode

from . import settings

from .signals import pre_restore, post_restore


if django.VERSION < (1, 9, 0):
    from django.db.models.query import ValuesQuerySet, ValuesListQuerySet


class BasePermanentQuerySet(QuerySet):
    def __deepcopy__(self, memo):
        obj = self.__class__(model=self.model)
        for k, v in self.__dict__.items():
            if k == '_result_cache':
                obj.__dict__[k] = None
            else:
                obj.__dict__[k] = copy.deepcopy(v, memo)
        return obj

    def __init__(self, *args, **kwargs):
        super(BasePermanentQuerySet, self).__init__(*args, **kwargs)

        self._unpatched = False

    def create(self, **kwargs):
        if not self._unpatched:
            if self.model.Permanent.restore_on_create and not kwargs.get(settings.FIELD):
                qs = self.get_unpatched()
                return qs.get_restore_or_create(**kwargs)
        return super(BasePermanentQuerySet, self).create(**kwargs)

    def get_restore_or_create(self, **kwargs):
        qs = self.get_unpatched()
        obj, created = qs.get_or_create(**kwargs)
        if isinstance(obj, dict):
            geter, seter = obj.get, obj.__setitem__
        else:
            geter, seter = partial(getattr, obj), partial(setattr, obj)

        if not created and geter(settings.FIELD, True):
            pre_restore.send(sender=self.model, instance=obj)
            seter(settings.FIELD, settings.FIELD_DEFAULT)
            self.model.all_objects.filter(id=geter('id')).update(**{settings.FIELD: settings.FIELD_DEFAULT})
            post_restore.send(sender=self.model, instance=obj)

        return obj

    def delete(self, force=False):
        """
        Deletes the records in the current QuerySet.
        """
        assert self.query.can_filter(), "Cannot use 'limit' or 'offset' with delete."

        del_query = self._clone()

        # The delete is actually 2 queries - one to find related objects,
        # and one to delete. Make sure that the discovery of related
        # objects is performed on the same database as the deletion.
        del_query._for_write = True

        # Disable non-supported fields.
        del_query.query.select_for_update = False
        del_query.query.select_related = False
        del_query.query.clear_ordering(force_empty=True)

        collector = Collector(using=del_query.db)
        collector.collect(del_query)
        if django.VERSION < (1, 9, 0):
            collector.delete(force=force)
        else:
            deleted, _rows_count = collector.delete(force=force)

        # Clear the result cache, in case this QuerySet gets reused.
        self._result_cache = None

        if django.VERSION < (1, 9, 0):
            return
        else:
            return deleted, _rows_count

    delete.alters_data = True

    def restore(self):
        return self.get_unpatched().update(**{settings.FIELD: settings.FIELD_DEFAULT})

    def values(self, *fields):
        if django.VERSION < (1, 9, 0):
            klass = type('CustomValuesQuerySet', (self.__class__, ValuesQuerySet,), {})
            return self._clone(klass=klass, setup=True, _fields=fields)

        return super(BasePermanentQuerySet, self).values(*fields)

    def values_list(self, *fields, **kwargs):
        if django.VERSION < (1, 9, 0):
            klass = type('CustomValuesListQuerySet', (self.__class__, ValuesListQuerySet,), {})
            clone = self._clone(klass=klass, setup=True, _fields=fields, **kwargs)

            if not hasattr(clone, "flat"):
                # Only assign flat if the clone didn't already get it from kwargs
                clone.flat = kwargs.get('flat')

            return clone

        return super(BasePermanentQuerySet, self).values_list(*fields, **kwargs)

    def _update(self, values):
        # Modifying trigger field have to effect all objects
        if settings.FIELD in [field.attname for field, _, _ in values] and not getattr(self, '_unpatched', False):
            return self.get_unpatched()._update(values)
        return super(BasePermanentQuerySet, self)._update(values)

    def get_unpatched(self):
        qs = self._clone()
        qs._unpatch()
        return qs

    def _clone(self, *args, **kwargs):
        c = super(BasePermanentQuerySet, self)._clone(*args, **kwargs)
        # We need clones stay unpatched
        if getattr(self, '_unpatched', False):
            c._unpatched = True
            c._unpatch()
        return c

    def _patch(self, q_object):
        self.query.add_q(q_object)

    def _unpatch(self):
        self._unpatched = True
        if not self.query.where.children:
            return
        condition = self.query.where.children[0]

        if django.VERSION < (1, 7, 0):
            is_patched = isinstance(condition, tuple) and condition[0].col == settings.FIELD
        elif django.VERSION < (1, 8, 0):
            is_patched = hasattr(condition, 'lhs') and condition.lhs.source.name == settings.FIELD
        else:
            is_patched = hasattr(condition, 'lhs') and condition.lhs.target.name == settings.FIELD
        if is_patched:
            del self.query.where.children[0]


class NonDeletedQuerySet(BasePermanentQuerySet):
    def __init__(self, *args, **kwargs):
        super(NonDeletedQuerySet, self).__init__(*args, **kwargs)
        if not self.query.where:
            self._patch(Q(**{settings.FIELD: settings.FIELD_DEFAULT}))


class DeletedWhereNode(WhereNode):
    pass


class DeletedQuerySet(BasePermanentQuerySet):
    def __init__(self, *args, **kwargs):
        super(DeletedQuerySet, self).__init__(*args, **kwargs)
        if not self.query.where:
            self.query.where_class = DeletedWhereNode
            self._patch(~Q(**{settings.FIELD: settings.FIELD_DEFAULT}))


class AllWhereNode(WhereNode):
    pass


class PermanentQuerySet(BasePermanentQuerySet):
    def __init__(self, *args, **kwargs):
        super(PermanentQuerySet, self).__init__(*args, **kwargs)
        if not self.query.where:
            self.query.where_class = AllWhereNode



from django.conf import settings


FIELD_CLASS = getattr(settings, 'PERMANENT_FIELD_CLASS', 'django.db.models.DateTimeField')
FIELD = getattr(settings, 'PERMANENT_FIELD', 'removed')
FIELD_KWARGS = getattr(settings, 'PERMANENT_FIELD_KWARGS', dict(
    default=None,
    null=True,
    blank=True,
    editable=False
))

FIELD_DEFAULT = FIELD_KWARGS['default']



import django
from django.db import models, router
from django.db.models.deletion import Collector
from . import settings
from .deletion import *  # NOQA
from .related import *  # NOQA
from .query import NonDeletedQuerySet, DeletedQuerySet, PermanentQuerySet
from .managers import QuerySetManager

from .signals import pre_restore, post_restore

if django.VERSION < (1, 9):
    from django.utils.module_loading import import_by_path as import_string
else:
    from django.utils.module_loading import import_string


class PermanentModel(models.Model):
    objects = QuerySetManager(NonDeletedQuerySet)
    deleted_objects = QuerySetManager(DeletedQuerySet)
    all_objects = QuerySetManager(PermanentQuerySet)
    if django.VERSION < (1, 10):
        _base_manager = QuerySetManager(NonDeletedQuerySet)

    class Meta:
        abstract = True

        if django.VERSION >= (1, 10):
            default_manager_name = 'objects'
            base_manager_name = 'objects'

    class Permanent:
        restore_on_create = False

    def delete(self, using=None, force=False):
        using = using or router.db_for_write(self.__class__, instance=self)
        assert self._get_pk_val() is not None, "%s object can't be deleted because its %s attribute is set to None." \
                                               % (self._meta.object_name, self._meta.pk.attname)
        collector = Collector(using=using)
        collector.collect([self])
        collector.delete(force=force)

    delete.alters_data = True

    def restore(self):
        pre_restore.send(sender=self.__class__, instance=self)
        setattr(self, settings.FIELD, settings.FIELD_DEFAULT)
        self.save(update_fields=[settings.FIELD])
        post_restore.send(sender=self.__class__, instance=self)


field = import_string(settings.FIELD_CLASS)
PermanentModel.add_to_class(settings.FIELD, field(**settings.FIELD_KWARGS))



from unittest import skipUnless

from django_permanent.signals import post_restore, pre_restore

import django
from django.db.models.signals import post_delete
from django.test import TestCase
from django.utils.timezone import now

from django_permanent.tests.test_app.models import RegularModel, RemovableRegularDepended
from .test_app.models import (
    CustomQsPermanent,
    M2MFrom,
    M2MTo,
    MyPermanentModel,
    MyPermanentModelWithManager,
    NonRemovableDepended,
    NonRemovableNullableDepended,
    RemovableNullableDepended,
    PermanentDepended,
    PermanentM2MThrough,
    RemovableDepended,
    RestoreOnCreateModel,
)


class TestDelete(TestCase):
    def setUp(self):
        self.permanent = MyPermanentModel.objects.create()

    def test_deletion(self):
        model = MyPermanentModel
        permanent2 = model.objects.create()
        self.permanent.delete()
        self.assertTrue(self.permanent.removed)
        self.assertEqual(list(model.objects.all()), [permanent2])
        self.assertEqual(list(model.all_objects.all()), [self.permanent, permanent2])
        self.assertEqual(list(model.deleted_objects.all()), [self.permanent])

    def test_depended(self):
        model = RemovableDepended
        model.objects.create(dependence=self.permanent)
        self.permanent.delete()
        self.assertEqual(list(model.objects.all()), [])

    def test_non_removable_depended(self):
        model = NonRemovableDepended
        depended = model.objects.create(dependence=self.permanent)
        self.permanent.delete()
        self.assertEqual(list(model.objects.all()), [depended])

    def test_non_removable_nullable_depended(self):
        model = NonRemovableNullableDepended
        model.objects.create(dependence=self.permanent)
        self.permanent.delete()
        depended = model.objects.first()
        self.assertEqual(depended.dependence, None)

    def test_removable_nullable_depended(self):
        model = RemovableNullableDepended
        model.objects.create(dependence=self.permanent)
        self.permanent.delete()
        depended = model.objects.first()
        self.assertEqual(depended.dependence, None)

    def test_remove_removable_nullable_depended(self):
        model = RemovableRegularDepended
        test_model = model.objects.create(dependence=RegularModel.objects.create(name='Test'))
        test_model.delete()
        try:
            self.assertIsNotNone(model.deleted_objects.first().dependence)
        except AttributeError:
            self.fail()

    def test_permanent_depended(self):
        model = PermanentDepended
        depended = model.objects.create(dependence=self.permanent)
        self.permanent.delete()
        self.assertEqual(list(model.objects.all()), [])
        self.assertEqual(list(model.deleted_objects.all()), [depended])
        new_depended = model.all_objects.get(pk=depended.pk)
        new_permanent = MyPermanentModel.all_objects.get(pk=self.permanent.pk)
        self.assertTrue(new_depended.removed)
        self.assertTrue(new_permanent.removed)
        self.assertEqual(new_depended.dependence_id, self.permanent.id)

    def test_related(self):
        p = PermanentDepended.objects.create(dependence=self.permanent)
        self.permanent.delete()
        self.assertEqual(list(PermanentDepended.all_objects.select_related('dependence').all()), [p])

    def test_double_delete(self):
        self.called = 0

        def post_delete_receiver(*args, **kwargs):
            self.called += 1

        post_delete.connect(post_delete_receiver, sender=PermanentDepended)

        model = PermanentDepended
        model.objects.create(dependence=self.permanent, removed=now())
        model.objects.create(dependence=self.permanent)
        self.permanent.delete()
        self.assertEqual(self.called, 1)

    def test_restore(self):
        self.called_pre = 0
        self.called_post = 0

        def pre_restore_receiver(sender, instance, **kwargs):
            self.called_pre += 1

        def post_restore_receiver(sender, instance, **kwargs):
            self.called_post += 1

        pre_restore.connect(pre_restore_receiver)
        post_restore.connect(post_restore_receiver)

        self.permanent.delete()
        self.permanent.restore()
        self.assertFalse(self.permanent.removed)
        self.assertEqual(list(MyPermanentModel.objects.all()), [self.permanent])

        pre_restore.disconnect(pre_restore_receiver)
        post_restore.disconnect(post_restore_receiver)

        self.assertEqual(self.called_pre, 1)
        self.assertEqual(self.called_post, 1)

    def test_forced_model_delete(self):
        self.permanent.delete(force=True)
        self.assertEqual(MyPermanentModel.all_objects.count(), 0)

    def test_forced_queryset_delete(self):
        MyPermanentModel.objects.all().delete(force=True)
        self.assertEqual(MyPermanentModel.all_objects.count(), 0)

    def test_forced_model_delete_removed(self):
        self.permanent.delete()
        self.permanent.delete(force=True)
        self.assertEqual(MyPermanentModel.all_objects.count(), 0)

    def test_forced_querset_delete_removed(self):
        self.permanent.delete()
        MyPermanentModel.all_objects.all().delete(force=True)
        self.assertEqual(MyPermanentModel.all_objects.count(), 0)


class TestIntegration(TestCase):
    def test_prefetch_bug(self):
        permanent1 = MyPermanentModel.objects.create()
        NonRemovableDepended.objects.create(dependence=permanent1)
        MyPermanentModel.objects.prefetch_related('nonremovabledepended_set').all()
        NonRemovableDepended.all_objects.prefetch_related('dependence').all()

    def test_related_manager_bug(self):
        permanent = MyPermanentModel.objects.create()
        PermanentDepended.objects.create(dependence=permanent)
        PermanentDepended.objects.create(dependence=permanent, removed=now())
        self.assertEqual(permanent.permanentdepended_set.count(), 1)
        self.assertEqual(PermanentDepended.objects.count(), 1)

    def test_m2m_manager(self):
        _from = M2MFrom.objects.create()
        _to = M2MTo.objects.create()
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to, removed=now())
        self.assertEqual(_from.m2mto_set.count(), 0)

    def test_m2m_manager_clear(self):
        _from = M2MFrom.objects.create()
        _to = M2MTo.objects.create()
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to)
        self.assertEqual(_from.m2mto_set.count(), 1)
        _from.m2mto_set.clear()
        self.assertEqual(_from.m2mto_set.count(), 0)
        self.assertEqual(PermanentM2MThrough.objects.count(), 0)
        self.assertEqual(PermanentM2MThrough.all_objects.count(), 1)
        self.assertEqual(M2MFrom.objects.count(), 1)
        self.assertEqual(M2MTo.objects.count(), 1)

    def test_m2m_manager_delete(self):
        _from = M2MFrom.objects.create()
        _to = M2MTo.objects.create()
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to)
        self.assertEqual(_from.m2mto_set.count(), 1)
        _from.m2mto_set.all().delete()
        self.assertEqual(_from.m2mto_set.count(), 0)
        self.assertEqual(PermanentM2MThrough.objects.count(), 0)
        self.assertEqual(PermanentM2MThrough.all_objects.count(), 1)
        self.assertEqual(M2MFrom.objects.count(), 1)
        self.assertEqual(M2MTo.objects.count(), 0)

    def test_m2m_prefetch_related(self):
        _from = M2MFrom.objects.create()
        _to = M2MTo.objects.create()
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to)
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to, removed=now())
        self.assertSequenceEqual(M2MFrom.objects.prefetch_related('m2mto_set').get(pk=_from.pk).m2mto_set.all(), [_to])
        self.assertEqual(M2MFrom.objects.prefetch_related('m2mto_set').get(pk=_from.pk).m2mto_set.count(), 1)

    @skipUnless(django.VERSION < (1, 8, 0), "Missing m2m")
    def test_m2m_select_related(self):
        _from = M2MFrom.objects.create()
        _to = M2MTo.objects.create()
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to)
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to, removed=now())
        self.assertSequenceEqual(M2MFrom.objects.select_related('m2mto_set').get(pk=_from.pk).m2mto_set.all(), [_to])
        self.assertEqual(M2MFrom.objects.select_related('m2mto_set').get(pk=_from.pk).m2mto_set.count(), 1)

    def test_m2m_all_objects(self):
        dependence = MyPermanentModel.objects.create(removed=now())
        depended = NonRemovableDepended.objects.create(dependence=dependence, removed=now())
        depended = NonRemovableDepended.all_objects.get(pk=depended.pk)
        self.assertEqual(depended.dependence, dependence)

    def test_m2m_deleted_through(self):
        _from = M2MFrom.objects.create()
        _to = M2MTo.objects.create()
        PermanentM2MThrough.objects.create(m2m_from=_from, m2m_to=_to, removed=now())
        self.assertEqual(M2MFrom.objects.filter(m2mto__id=_to.pk).count(), 0)


class TestPassThroughManager(TestCase):
    def test_pass_through_manager(self):
        self.assertTrue(hasattr(CustomQsPermanent.objects, 'test'))
        self.assertTrue(hasattr(CustomQsPermanent.objects, 'restore'))
        self.assertTrue(CustomQsPermanent.objects.get_restore_or_create(id=10))


class TestCustomQSMethods(TestCase):
    def test__get_restore_or_create__get(self):
        self.obj = MyPermanentModel.objects.create(name="old")
        self.assertEqual(MyPermanentModel.objects.get_restore_or_create(name="old").id, 1)

    def test__get_restore_or_create__restore(self):
        self.called_pre = 0
        self.called_post = 0

        def pre_restore_receiver(sender, instance, **kwargs):
            self.called_pre += 1

        def post_restore_receiver(sender, instance, **kwargs):
            self.called_post += 1

        pre_restore.connect(pre_restore_receiver)
        post_restore.connect(post_restore_receiver)

        obj = MyPermanentModel.objects.create(name="old", removed=now())
        self.assertEqual(MyPermanentModel.objects.get_restore_or_create(name="old").id, obj.id)
        self.assertEqual(MyPermanentModel.objects.count(), 1)
        self.assertEqual(MyPermanentModel.all_objects.count(), 1)

        pre_restore.disconnect(pre_restore_receiver)
        post_restore.disconnect(post_restore_receiver)

        self.assertEqual(self.called_pre, 1)
        self.assertEqual(self.called_post, 1)

    def test__get_restore_or_create__create(self):
        MyPermanentModel.objects.get_restore_or_create(name="old")
        self.assertEqual(MyPermanentModel.objects.get_restore_or_create(name="old").id, 1)
        self.assertEqual(MyPermanentModel.objects.count(), 1)
        self.assertEqual(MyPermanentModel.all_objects.count(), 1)

    def test_restore(self):
        MyPermanentModel.objects.create(name="old", removed=now())
        MyPermanentModel.objects.filter(name="old").restore()
        self.assertEqual(MyPermanentModel.objects.count(), 1)
        self.assertEqual(MyPermanentModel.all_objects.count(), 1)

    def test_restore_on_create(self):
        MyPermanentModel.Permanent.restore_on_create = True
        first = MyPermanentModel.objects.create(name='unique', removed=now())
        second = MyPermanentModel.objects.create(name='unique')
        self.assertEqual(first, second)
        MyPermanentModel.Permanent.restore_on_create = False


class TestCustomManager(TestCase):
    def setUp(self):
        MyPermanentModelWithManager.objects.create(name="regular")
        MyPermanentModelWithManager.objects.create(name="removed", removed=now())

    def test_custom_method(self):
        MyPermanentModelWithManager.objects.test()

    def test_non_removed(self):
        self.assertEqual(MyPermanentModelWithManager.objects.count(), 1)

    def test_removed(self):
        self.assertEqual(MyPermanentModelWithManager.objects.count(), 1)

    def test_all(self):
        self.assertEqual(MyPermanentModelWithManager.any_objects.count(), 2)


class RetoreOnCreateTestCase(TestCase):
    def setUp(self):
        self.obj = RestoreOnCreateModel.objects.create(name='obj1')

    def test_restore_on_create(self):
        self.obj.delete()
        new_obj = RestoreOnCreateModel.objects.create(name='obj1')

        self.assertEqual(new_obj.pk, self.obj.pk)



# -*- coding: utf-8 -*-
from collections import Counter
from functools import partial
from operator import attrgetter

from django.db import transaction
from django.db.models import signals, sql
from django.db.models.deletion import Collector
from django.utils import six
from django.utils.timezone import now
from django import VERSION as DJANGO_VERSION

from .settings import FIELD


def delete(self, force=False):
    """
        Patched the BaseCollector.delete with soft delete support for PermanentModel
    """
    from .models import PermanentModel
    time = now()
    deleted_counter = Counter()

    # sort instance collections
    for model, instances in self.data.items():
        self.data[model] = sorted(instances, key=attrgetter("pk"))

    # if possible, bring the models in an order suitable for databases that
    # don't support transactions or cannot defer constraint checks until the
    # end of a transaction.
    self.sort()
    # number of objects deleted for each model label
    deleted_counter = Counter()

    if DJANGO_VERSION < (1, 8, 0):
        transaction_handling = partial(transaction.commit_on_success_unless_managed, using=self.using)
    else:
        transaction_handling = partial(transaction.atomic, using=self.using, savepoint=False)

    with transaction_handling():
        # send pre_delete signals
        for model, obj in self.instances_with_model():
            if not model._meta.auto_created:
                signals.pre_delete.send(
                    sender=model, instance=obj, using=self.using
                )

        # fast deletes
        for qs in self.fast_deletes:
            if issubclass(qs.model, PermanentModel) and not force:  # Update PermanentModel instance
                pk_list = [obj.pk for obj in qs]
                qs = sql.UpdateQuery(qs.model)
                qs.update_batch(pk_list, {FIELD: time}, self.using)
                count = len(pk_list)
            else:
                count = qs._raw_delete(using=self.using)

            if DJANGO_VERSION >= (1, 9, 0):
                deleted_counter[qs.model._meta.label] += count

        # update fields
        for model, instances_for_fieldvalues in six.iteritems(self.field_updates):
            query = sql.UpdateQuery(model)
            for (field, value), instances in six.iteritems(instances_for_fieldvalues):
                query.update_batch([obj.pk for obj in instances],
                                   {field.name: value}, self.using)

        # reverse instance collections
        for instances in six.itervalues(self.data):
            instances.reverse()

        # delete instances
        for model, instances in six.iteritems(self.data):
            pk_list = [obj.pk for obj in instances]
            if issubclass(model, PermanentModel) and not force:
                query = sql.UpdateQuery(model)
                query.update_batch(pk_list, {FIELD: time}, self.using)
                for instance in instances:
                    setattr(instance, FIELD, time)
                count = len(pk_list)
            else:
                query = sql.DeleteQuery(model)
                count = query.delete_batch(pk_list, self.using)

            if DJANGO_VERSION >= (1, 9, 0):
                deleted_counter[model._meta.label] += count

            if not model._meta.auto_created:
                for obj in instances:
                    signals.post_delete.send(
                        sender=model, instance=obj, using=self.using
                    )

    # update collected instances
    for model, instances_for_fieldvalues in six.iteritems(self.field_updates):
        for (field, value), instances in six.iteritems(instances_for_fieldvalues):
            for obj in instances:
                setattr(obj, field.attname, value)
    for model, instances in six.iteritems(self.data):
        for instance in instances:
            if issubclass(model, PermanentModel) and not force:
                continue
            setattr(instance, model._meta.pk.attname, None)

    if DJANGO_VERSION >= (1, 9, 0):
        return sum(deleted_counter.values()), dict(deleted_counter)


Collector.delete = delete



import django
from django.db.models.fields.related import ForeignObject

from . import settings
from .query import AllWhereNode, DeletedWhereNode


def get_extra_restriction_patch(func):
    def wrapper(self, where_class, alias, lhs):
        cond = func(self, where_class, alias, lhs)

        from .models import PermanentModel
        if not issubclass(self.model, PermanentModel) or issubclass(where_class, AllWhereNode):
            return cond

        if issubclass(where_class, DeletedWhereNode):
            cond = cond or ~where_class()
        else:
            cond = cond or where_class()

        if django.VERSION < (1, 8, 0):
            field = self.model._meta.get_field_by_name(settings.FIELD)[0]
        else:
            field = self.model._meta.get_field(settings.FIELD)

        if django.VERSION < (1, 7, 0):
            from django.db.models.sql.where import Constraint
            if settings.FIELD_DEFAULT is None:
                lookup = Constraint(lhs, settings.FIELD, field), 'isnull', True
            else:
                lookup = Constraint(lhs, alias, field), 'exact', settings.FIELD_DEFAULT

        else:
            if django.VERSION < (1, 8, 0):
                from django.db.models.sql.datastructures import Col
            else:
                from django.db.models.expressions import Col

            if settings.FIELD_DEFAULT is None:
                lookup = field.get_lookup('isnull')(Col(lhs, field, field), True)
            else:
                lookup = field.get_lookup('exact')(Col(lhs, field, field), settings.FIELD_DEFAULT)

        cond.add(lookup, 'AND')

        return cond
    return wrapper


ForeignObject.get_extra_restriction = get_extra_restriction_patch(ForeignObject.get_extra_restriction)


if django.VERSION > (1, 8, -1):
    if django.VERSION > (1, 9, 0):
        from django.db.models.fields.related_descriptors import ForwardManyToOneDescriptor as Descriptor
    else:
        from django.db.models.fields.related import ReverseSingleRelatedObjectDescriptor as Descriptor

    def get_queryset_patch(func):
        def wrapper(self, **hints):
            from .models import PermanentModel
            instance = hints.get('instance')
            if instance and isinstance(instance, PermanentModel) and getattr(instance, settings.FIELD):
                if django.VERSION < (1, 9, 0):
                    model = self.field.rel.to
                else:
                    model = self.field.remote_field.model
                if hasattr(model, 'all_objects'):
                    return model.all_objects
                return model.objects
            return func(self, **hints)
        return wrapper

    Descriptor.get_queryset = get_queryset_patch(Descriptor.get_queryset)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

tests_require = ["Django>=1.6.0"]

try:
    import django
    if django.VERSION < (1, 7, 0):
        tests_require.append("django-model-utils==2.3.1")
except ImportError:
    pass

setup(
    name='django-permanent-model',
    version='1.1.7',
    description='Yet another approach to provide soft (logical) delete or masking (thrashing) django models instead of deleting them physically from db.',
    author='Mikhail Antonov',
    author_email='atin65536@gmail.com',
    long_description=open('README.rst').read(),
    url='https://github.com/MnogoByte/django-permanent',
    packages=find_packages(),
    install_requires=["Django>=1.6.0"],
    keywords=['django', 'delete', 'undelete', 'safedelete', 'remove', 'restore', 'softdelete', 'logicaldelete', 'trash'],
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: BSD License",
    ],
    tests_require=tests_require,
    test_suite='runtests.runtests',
    license="BSD"
)



from django.db import models
from django.db.models import Model

from ...models import PermanentModel
from ...managers import MultiPassThroughManager
from ...query import DeletedQuerySet, PermanentQuerySet, NonDeletedQuerySet


class BaseTestModel(Model):
    class Meta:
        abstract = True

    def __str__(self):
        return str(self.pk)


class MyPermanentModel(PermanentModel, BaseTestModel):
    name = models.CharField(max_length=255, blank=True, null=True)
    pass


class RegularModel(BaseTestModel):
    name = models.CharField(max_length=255, blank=True, null=True)
    pass


class RemovableRegularDepended(PermanentModel, BaseTestModel):
    dependence = models.ForeignKey(RegularModel, on_delete=models.CASCADE)


class RemovableDepended(BaseTestModel):
    dependence = models.ForeignKey(MyPermanentModel, on_delete=models.CASCADE)


class NonRemovableDepended(PermanentModel, BaseTestModel):
    dependence = models.ForeignKey(MyPermanentModel, on_delete=models.DO_NOTHING)


class NonRemovableNullableDepended(PermanentModel, BaseTestModel):
    dependence = models.ForeignKey(MyPermanentModel, on_delete=models.SET_NULL, null=True)


class RemovableNullableDepended(PermanentModel, BaseTestModel):
    dependence = models.ForeignKey(MyPermanentModel, on_delete=models.SET_NULL, null=True)


class PermanentDepended(PermanentModel, BaseTestModel):
    dependence = models.ForeignKey(MyPermanentModel, on_delete=models.CASCADE)


class M2MFrom(BaseTestModel):
    pass


class PermanentM2MThrough(PermanentModel):
    m2m_from = models.ForeignKey('M2MFrom', on_delete=models.CASCADE)
    m2m_to = models.ForeignKey('M2MTo', on_delete=models.CASCADE)


class M2MTo(BaseTestModel):
    m2m_from = models.ManyToManyField('M2MFrom', through=PermanentM2MThrough)


class MyPermanentQuerySet(PermanentQuerySet):
    def test(self):
        pass


class MyPermanentModelWithManager(PermanentModel, BaseTestModel):
    name = models.CharField(max_length=255, blank=True, null=True)

    objects = MultiPassThroughManager(MyPermanentQuerySet, NonDeletedQuerySet)
    deleted_objects = MultiPassThroughManager(MyPermanentQuerySet, DeletedQuerySet)
    any_objects = MultiPassThroughManager(MyPermanentQuerySet, PermanentQuerySet)


class TestQS(object):
    def test(self):
        return "ok"


class CustomQsPermanent(PermanentModel, BaseTestModel):
    objects = MultiPassThroughManager(TestQS, DeletedQuerySet)


class RestoreOnCreateModel(PermanentModel, BaseTestModel):
    name = models.CharField(max_length=255, blank=True, null=True)

    class Permanent:
        restore_on_create = True
